/* $Header: solver.h,v 3.3 98/09/03 13:26:00 carl Exp $ */

/************************************************************************
 * Copyright (c) 1998 The Trustees of Princeton University.
 * All rights reserved.
 * 
 * PRINCETON UNIVERSITY MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF
 * THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 * OR IMPLIED WARRANTY.  PRINCETON UNIVERSITY SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY THE USERS OF THIS SOFTWARE.
 ************************************************************************/


#ifndef _SOLVER_INCLUDED
#define _SOLVER_INCLUDED

#include "minopt.h"
#include "solutions.h"

/* Name Length for the solvers */
#define SOLVERNAMELEN 16

/* Capabilities */
#define LPTYPE          0x0010    /* Can solve LP */
#define MILPTYPE        0x0020    /* Can solve MILP */
#define NLPTYPE         0x0040    /* Can solve NLP */
#define OCPTYPE         0x0080    /* Can solve OC (Optimal control) */
#define MINLPTYPE       0x0100    /* Can solve MINLP */
#define DYNTYPE         0x0200    /* Can solve Dynamic */

/* Capability of each solver */
#define CPLEXCAP        (LPTYPE | MILPTYPE)
#define LPSOLVECAP      (LPTYPE | MILPTYPE)
#define MINOSCAP        (LPTYPE | NLPTYPE)
#define SNOPTCAP        (LPTYPE | NLPTYPE)
#define NPSOLCAP        (LPTYPE | NLPTYPE)
#define FSQPCAP         (LPTYPE | NLPTYPE)
#define DAEOPTCAP       (OCPTYPE)
#define CPNPCAP         (MINLPTYPE)
#define GBDCAP          (MINLPTYPE)
#define OAERCAP         (MINLPTYPE)
#define OAERAPCAP       (MINLPTYPE)
#define GCDCAP          (MINLPTYPE)
#define DASOLVCAP       (DYNTYPE)
#define DAESSACAP       (DYNTYPE)
#define DASPKSOCAP      (DYNTYPE)

/* Define the default solvers */
/* first, if the solver is defined, set its capabilities
   and then if a solver with the certain capabilities has not yet
   been defined, define the solver as the one to be used for
   the certain problem.  */

#ifdef CPLEX
#  define CPLEXSOLV     1
#  define CPLEX_(f)     (f)
#  ifndef LPDEF
#    define LPDEF       CPLEXSOLV
#  endif
#  ifndef MILPDEF
#    define MILPDEF     CPLEXSOLV
#  endif
#else
#  define CPLEXSOLV     0
#  define CPLEX_(f)     (NULL)
#endif

#ifdef LPSOLVE
#  define LPSOLVESOLV   2
#  define LPSOLVE_(f)   (f)
#  ifndef LPDEF
#    define LPDEF       LPSOLVESOLV
#  endif
#  ifndef MILPDEF
#    define MILPDEF     LPSOLVESOLV
#  endif
#else
#  define LPSOLVESOLV   0
#  define LPSOLVE_(f)   (NULL)
#endif

#ifdef MINOS
#  define MINOSSOLV     3
#  define MINOS_(f)     (f)
#  ifndef NLPDEF
#    define NLPDEF      MINOSSOLV
#  endif
#  ifndef LPDEF
#    define LPDEF       MINOSSOLV
#  endif
#else
#  define MINOSSOLV     0
#  define MINOS_(f)     NULL
#endif

#ifdef SNOPT
#  define SNOPTSOLV     4
#  define SNOPT_(f)     (f)
#  ifndef NLPDEF
#    define NLPDEF      SNOPTSOLV
#  endif
#else
#  define SNOPTSOLV     0
#  define SNOPT_(f)     NULL
#endif

#ifdef NPSOL
#  define NPSOLSOLV     5
#  define NPSOL_(f)     (f)
#  ifndef NLPDEF
#    define NLPDEF      NPSOLSOLV
#  endif
#  ifndef LPDEF
#    define LPDEF       NPSOLSOLV
#  endif
#else
#  define NPSOLSOLV     0
#  define NPSOL_(f)     (NULL)
#endif

#ifdef FSQP
#  define FSQPSOLV      6
#  define FSQP_(f)      (f)
#  ifndef NLPDEF
#    define NLPDEF      FSQPSOLV
#  endif
#  ifndef LPDEF
#    define LPDEF       FSQPSOLV
#  endif
#else
#  define FSQPSOLV      0
#  define FSQP_(f)      NULL
#endif

#ifdef DAEOPT
#  define DAEOPTSOLV    7
#  define DAEOPT_(f)    (f)
#  ifndef OCPDEF
#    define OCPDEF      DAEOPTSOLV
#  endif
#else
#  define DAEOPTSOLV    0
#  define DAEOPT_(f)    NULL
#endif

/* define the MINLP solvers if we have solvers for the subproblems */
#if defined(NLPDEF) && defined(MILPDEF)
#  define GBDSOLV       8
#  define GBD_(f)       (f)
#  define OAERSOLV      9
#  define OAER_(f)      (f)
#  define OAERAPSOLV    10
#  define OAERAP_(f)    (f)
/* GCD is not available right now */
#  define GCDSOLV       0
#  define GCD_(f)       NULL
/* MAKE GBD the default solver */
#  define MINLPDEF      GBDSOLV
#else
#  define GBDSOLV       0
#  define GBD_(f)       NULL
#  define OAERSOLV      0
#  define OAER_(f)      NULL
#  define OAERAPSOLV    0
#  define OAERAP_(f)    NULL
#  define GCDSOLV       0
#  define GCD_(f)       NULL
#endif

#ifdef DASOLV
#  define DASOLVSOLV    12
#  define DASOLV_(f)    (f)
#  ifndef DYNDEF
#    define DYNDEF      DASOLVSOLV
#  endif
#else
#  define DASOLVSOLV    0
#  define DASOLV_(f)    NULL
#endif

#ifdef DAESSA
#  define DAESSASOLV    13
#  define DAESSA_(f)    (f)
#  ifndef DYNDEF
#    define DYNDEF      DAESSASOLV
#  endif
#else
#  define DAESSASOLV     0
#  define DAESSA_(f)     NULL
#endif

#ifdef DASPKSO
#  define DASPKSOSOLV   14
#  define DASPKSO_(f)   (f)
#  ifndef DYNDEF
#    define DYNDEF      DASPKSOSOLV
#  endif
#else
#  define DASPKSOSOLV   0
#  define DASPKSO_(f)   NULL
#endif

#ifndef LPDEF
#  define LPDEF         0
#endif
#ifndef NLPDEF
#  define NLPDEF        0
#endif
#ifndef MILPDEF
#  define MILPDEF       0
#endif
#ifndef OCPDEF
#  define OCPDEF        0
#endif
#ifndef MINLPDEF
#  define MINLPDEF      0
#endif
#ifndef DYNDEF
#  define DYNDEF        0
#endif

/* define DEFSOLV to have the DEFault SOLvers */
#define DEFSOLV  ((LPDEF)|(NLPDEF << 4)|(MILPDEF << 8)|(OCPDEF << 12)|(MINLPDEF << 16) | (DYNDEF << 20))

#define DOIHAVE(s)      (s)                     /* Check availability */
#define CANITDO(s,p)    (s & p)                 /* Check capability */
#define FSOLV(p,n)      SOLV(p,n)
#define SOLV(a,b)       (a ## _ ## b)

/* Set and get different type of solvers */
#define SETDEFSOLV(s)    (DEC_SOLV = (long int) s)
#define SET_LPSOLV(s)    (DEC_SOLV = ((DEC_SOLV & 0xfffffff0L) | (s)))
#define GET_LPSOLV()     (DEC_SOLV & 0x00000000fL)
#define SET_NLPSOLV(s)   (DEC_SOLV = ((DEC_SOLV & 0xffffff0fL) | (s) << 4))
#define GET_NLPSOLV()    ((DEC_SOLV & 0x000000f0L) >> 4)
#define SET_MILPSOLV(s)  (DEC_SOLV = ((DEC_SOLV & 0xfffff0ffL) | (s) << 8))
#define GET_MILPSOLV()   ((DEC_SOLV & 0x00000f00L) >> 8)
#define SET_OCPSOLV(s)   (DEC_SOLV = ((DEC_SOLV & 0xffff0fffL) | (s) << 12))
#define GET_OCPSOLV()    ((DEC_SOLV & 0x0000f000L) >> 12)
#define SET_MINLPSOLV(s) (DEC_SOLV = ((DEC_SOLV & 0xfff0ffffL) | (s) << 16))
#define GET_MINLPSOLV()  ((DEC_SOLV & 0x000f0000L) >> 16)
#define SET_DYNSOLV(s)   (DEC_SOLV = ((DEC_SOLV & 0xff0fffffL) | (s) << 20))
#define GET_DYNSOLV()    ((DEC_SOLV & 0x00f00000L) >> 20)

#if defined(__STDC__) || defined(__cplusplus)
# define P_(s) s
#else
# define P_(s) ()
#endif

typedef OPTSOLTYPE (*minfuntype) P_((PROBLEMTYPE p, double *x, double *y));
typedef PRISOLTYPE (*prifuntype) P_((PROBLEMTYPE p, int i, double *x, double *y));
typedef PRISOLTYPE (*rlxfuntype) P_((PROBLEMTYPE p, int i, double *x, double *y));
typedef MASSOLTYPE (*masfuntype) P_((PROBLEMTYPE p, int i, PRISOLTYPE psol));
typedef PRISOLTYPE (*dmsfuntype) P_((PROBLEMTYPE p, int i, PRISOLTYPE psol));
typedef DYNSOLTYPE (*dynfuntype) P_((PROBLEMTYPE p, double *x, int npoints, double *time, int sens));
typedef DYNSOLTYPE (*adjfuntype) P_((PROBLEMTYPE p, DYNSOLTYPE dynsol, double *lambda, int npoints, double *time));

void MINOPT_print_solvers P_((FILE *fp));
void set_solver_defaults P_((void));
int set_solver P_((char *problem, char *solver));
char *get_solver_string P_((int solver));
int get_solver P_((int type));
prifuntype get_prifun P_((int problem, int type, char *sname));
masfuntype get_masfun P_((int problem, int type, char *sname));
dmsfuntype get_dmsfun P_((int problem, int type, char *sname));
rlxfuntype get_rlxfun P_((int problem, int type, char *sname));
minfuntype get_minfun P_((int problem, int type, char *sname));
dynfuntype get_dynfun P_((int problem, int type, char *sname));
adjfuntype get_adjfun P_((int problem, int type, char *sname));
#undef P_

#endif /* _SOLVER_INCLUDED */
